@@ -2,6 +2,7 @@
 #' @description Input Bib file with complete journal, output Bib file after abbreviation of journal, and return to the abbreviation table of journal
 #' @importFrom rlang is_empty
 #' @importFrom stringr str_trim str_to_lower str_replace_all str_extract str_replace
+#' @importFrom stringi stri_unescape_unicode
 #' @importFrom stats complete.cases
 #' @importFrom dplyr left_join
 #' @importFrom httr GET
@@ -59,6 +60,12 @@ abbr2bib <- function(file, outfile = tempfile(fileext = ".bib")) {
     temp = str_trim(str_to_lower(x),side = 'both')
     gsub("[\t ]{2,}"," ", temp)
   })
+  # # Unicode to UTF-8
+  # library(data.table)
+  # library(stringi)
+  # abbrTable = data.table::as.data.table(abbrTable)
+  # abbrTable = abbrTable[,lapply(.SD, function(x)stringi::stri_unescape_unicode(x))]
+  abbrTable = as.data.frame(lapply(abbrTable, function(x)stringi::stri_unescape_unicode(x)))
   abbrTableSub = tibble::as_tibble(abbrTable)
   abbrTableSub = abbrTableSub[,c("journal_lower",'journal_abbr','originFile')]
   tib = dplyr::left_join(item_tib, abbrTableSub, by = "journal_lower")
